#![feature(linked_list_remove)]
#![feature(label_break_value)]

#![allow(dead_code)]

use termprogress::{
    ProgressBar,
};

#[macro_use]
mod ext;
use ext::*;

mod config;
mod args;
mod url;
mod error;
mod tempfile;
mod loli;

#[cfg(feature="async")]
mod work_async;
#[cfg(not(feature="async"))]
mod work;

//mod list;

pub fn parse_args() -> Result<config::Config, args::Error>
{
    match args::parse_args()? {
	args::Mode::Normal(conf) => Ok(conf),
	args::Mode::Help => args::usage(),
    }
}

#[cfg(feature="async")]
#[cfg_attr(feature="async", tokio::main)]
async fn main() -> Result<(), Box<dyn std::error::Error>>
{
    let conf = match parse_args() {
	Ok(v) => v,
	Err(e) => {
	    println!("Failed to parse args: {}", e);
	    std::process::exit(1)
	},
    };

    work_async::work(conf).await
}

#[cfg(not(feature="async"))]
fn main() -> Result<(), Box<dyn std::error::Error>>
{
    let conf = parse_args()?;

    work::work(conf)
}
